import {select, csv, scaleLinear, scaleTime, extent, axisLeft, axisBottom, area, curveBasis, timeFormat} from 'd3';
// initialize the svg
const svg = select('svg');
// define dimensions of svg
const width = +svg.attr('width');
const height = +svg.attr('height');

// method that takes data as parameter and creates a area chart
const render = data => {
  const title = 'Google Stock Price';
  const xAxisTickFormat = timeFormat('%b %Y');  // x axis date format
  const xValue = d => d.date;   // x axis tick values
  const xAxisLabel = 'Months';  // x axis label
  
  const yValue = d => d.close_price;   //y axis value
  const circleRadius = 3;
  const yAxisLabel = 'Closing Price ($)';  //y axis label
  
//   define margins for the graph
  const margin = { top: 60, right: 40, bottom: 110, left: 105 };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
//   define x, vertical lines according to width
  const xScale = scaleTime().domain(extent(data, xValue)).range([0, innerWidth]);
//   define y, horizontal lines according to height  
  const yScale = scaleLinear().domain(extent(data, yValue)).range([innerHeight, 0]).nice();
//   initialize g that will group svg emelents
  const g = svg.append('g').attr('transform', `translate(${margin.left},${margin.top})`);
  
//   set the x axis tick values with xscale and xAxisTickFormat
  const xAxis = axisBottom(xScale).ticks(15).tickSize(-innerHeight)
  	.tickFormat(xAxisTickFormat).tickPadding(15);
// set y axis tick values yScale 
  const yAxis = axisLeft(yScale).tickSize(-innerWidth).tickPadding(10);
//   append yaxis to group
  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();
//   format and style the yAxis labels
  yAxisG.append('text').attr('class', 'axis-label').attr('y', -80).attr('x', -innerHeight / 2)
      .attr('fill', 'black').attr('transform', `rotate(-90)`).attr('text-anchor', 'middle').text(yAxisLabel);
//   append xaxis to group
  const xAxisG = g.append('g').call(xAxis).attr('transform', `translate(0,${innerHeight})`);
  
//   rotate x axis ticks
  xAxisG.selectAll("text").attr("transform", "translate(-10,0) rotate(-30)").style("text-anchor", "end");

// generate the area according x and y values  
  const areaGenerator = area().x(d => xScale(xValue(d))).y0(innerHeight)
    .y1(d => yScale(yValue(d))).curve(curveBasis);
  
//   append the path of the area to the group
  g.append('path').attr('class', 'line-path').attr('d', areaGenerator(data));

//   append title to the group
  g.append('text').attr('class', 'title').attr('y', -10).text(title);

// append x label to the group
  xAxisG.append('text').attr('class', 'axis-label').attr('y', 100)
      .attr('x', innerWidth / 2).attr('fill', 'black').text(xAxisLabel);  
};

/*
 * the following block gets the raw csv data and  
 * returns the arrays of data that is month and stock price of data
 */
const csvUrl = "https://gist.githubusercontent.com/snlgrglns/35fa67ee4fb85e2159851382335322c2/raw/c9238af54708055473f9c3951c4e811f30caded1/google_stock_price.csv";
csv(csvUrl)
  .then(data => {
    data.forEach(d => {
      d.date = new Date(d.Date);
      d.close_price = +d.Close;
    });
    render(data);
  });
